import transaction
from io import BytesIO
from pyramid.compat import bytes_
from ptah.testing import PtahTestCase


class TestBlob(PtahTestCase):

    def test_blob(self):
        import ptah.cms

        blob = ptah.cms.blob_storage.add(BytesIO(bytes_('blob data','utf-8')))
        self.assertTrue(ptah.cms.IBlob.providedBy(blob))
        self.assertEqual(blob.read(), bytes_('blob data','utf-8'))
        self.assertTrue(ptah.cms.IBlobStorage.providedBy(ptah.cms.blob_storage))

    def test_blob_create(self):
        import ptah.cms

        blob = ptah.cms.blob_storage.create()
        self.assertTrue(ptah.cms.IBlob.providedBy(blob))
        self.assertEqual(blob.read(), None)

    def test_blob_metadata(self):
        import ptah.cms

        blob = ptah.cms.blob_storage.add(
            BytesIO(bytes_('blob data','utf-8')),
            filename='test.txt', mimetype='text/plain')

        self.assertEqual(blob.filename, 'test.txt')
        self.assertEqual(blob.mimetype, 'text/plain')

    def test_blob_info(self):
        import ptah.cms
        blob = ptah.cms.blob_storage.add(
            BytesIO(bytes_('blob data','utf-8')),
            filename='test.txt', mimetype='text/plain')

        info = blob.info()
        self.assertEqual(info['__uri__'], blob.__uri__)
        self.assertEqual(info['filename'], 'test.txt')
        self.assertEqual(info['mimetype'], 'text/plain')

    def test_blob_resolver(self):
        import ptah

        blob = ptah.cms.blob_storage.add(BytesIO(bytes_('blob data','utf-8')))

        blob_uri = blob.__uri__
        transaction.commit()

        blob = ptah.resolve(blob_uri)

        self.assertEqual(blob.__uri__, blob_uri)
        self.assertEqual(blob.read(), bytes_('blob data','utf-8'))

    def test_blob_with_parent(self):
        import ptah

        class MyContent(ptah.cms.Node):
            __name__ = ''
            __mapper_args__ = {'polymorphic_identity': 'mycontent'}
            __uri_factory__ = ptah.UriFactory('test')

        content = MyContent()
        content_uri = content.__uri__
        ptah.get_session().add(content)

        blob_uri = ptah.cms.blob_storage.add(
            BytesIO(bytes_('blob data','utf-8')), content).__uri__
        transaction.commit()

        blob = ptah.resolve(blob_uri)
        self.assertEqual(blob.__parent_ref__.__uri__, content_uri)

        blob = ptah.cms.blob_storage.getByParent(content_uri)
        self.assertEqual(blob.__uri__, blob_uri)

    def test_blob_write(self):
        import ptah

        blob_uri = ptah.cms.blob_storage.add(
            BytesIO(bytes_('blob data','utf-8'))).__uri__
        blob = ptah.resolve(blob_uri)
        blob.write(bytes_('new data','utf-8'))
        transaction.commit()

        blob = ptah.resolve(blob_uri)
        self.assertEqual(blob.read(), bytes_('new data','utf-8'))

    def test_blob_rest_data(self):
        import ptah.cms
        from ptah.cms.rest import blobData

        blob = ptah.cms.blob_storage.add(
            BytesIO(bytes_('blob data','utf-8')),
            filename='test.txt', mimetype='text/plain')

        response = blobData(blob, self.request)
        self.assertEqual(response.body, bytes_('blob data','utf-8'))
        self.assertEqual(
            response.headerlist,
            [('Content-Disposition', bytes_('filename="test.txt"','utf-8')),
             ('Content-Length', '9')])



import sqlalchemy as sqla
import ptah
from ptah.populate import POPULATE_ID, Populate
from pyramid import testing
from pyramid.exceptions import ConfigurationConflictError


class TestPopulateDirective(ptah.PtahTestCase):

    _init_ptah = False

    def test_step_registration(self):
        import ptah

        @ptah.populate('step', title='Step', requires=['test-dep'])
        def step(registry):
            """ """

        self.init_ptah()

        data = ptah.get_cfg_storage(POPULATE_ID)

        self.assertIn('step', data)
        self.assertIs(data['step']['factory'], step)
        self.assertEqual(data['step']['title'], 'Step')
        self.assertTrue(data['step']['active'])
        self.assertEqual(data['step']['requires'], ['test-dep'])

    def test_step_pyramid_registration(self):
        import ptah

        def step(registry):
            """ """

        config = testing.setUp()
        config.include('ptah')
        config.ptah_populate_step('step', factory=step,
                                  title='Test', active=False)
        config.commit()

        data = config.get_cfg_storage(POPULATE_ID)

        self.assertIn('step', data)
        self.assertIs(data['step']['factory'], step)
        self.assertEqual(data['step']['title'], 'Test')
        self.assertFalse(data['step']['active'])
        self.assertEqual(data['step']['requires'], ())

    def test_step_registration_conflicts(self):
        import ptah

        @ptah.populate('step')
        @ptah.populate('step')
        def step(registry):
            """ """

        self.assertRaises(ConfigurationConflictError, self.init_ptah)


class TestPyramidDrective(ptah.PtahTestCase):

    def test_directive_execute(self):
        data = [False, False]
        def step1(registry):
            data[0] = True

        def step2(registry): # pragma: no cover
            data[0] = True

        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, factory=step1)

        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        self.config.ptah_populate()

        self.assertTrue(data[0])
        self.assertFalse(data[1])

    def test_directive_execute_populate_mode(self):
        data = [False]
        def step(registry): # pragma: no cover
            data[0] = True

        self.config.ptah_populate_step(
            'custom-step', title='Custom step',
            active=True, factory=step)

        import ptah
        ptah.POPULATE = True

        self.config.ptah_populate()

        ptah.POPULATE = False

        self.assertFalse(data[0])


class TestListSteps(ptah.PtahTestCase):

    def test_list_simple(self):
        def step1(registry):
            """ """
        def step2(registry):
            """ """

        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, factory=step1)

        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        steps = Populate(self.registry).list_steps()
        steps = dict((s['name'], s) for s in steps)

        self.assertIn('custom-step1', steps)
        self.assertNotIn('custom-step2', steps)
        self.assertEqual(steps['custom-step1']['factory'], step1)

    def test_list_all(self):
        def step1(registry):
            """ """
        def step2(registry):
            """ """

        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, factory=step1)

        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        steps = Populate(self.registry).list_steps(all=True)
        steps = dict((s['name'], s) for s in steps)

        self.assertIn('custom-step1', steps)
        self.assertIn('custom-step2', steps)
        self.assertEqual(steps['custom-step1']['factory'], step1)
        self.assertEqual(steps['custom-step2']['factory'], step2)

    def test_list_explicit(self):
        def step1(registry):
            """ """
        def step2(registry):
            """ """

        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, factory=step1)

        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        steps = Populate(self.registry).list_steps(('custom-step2',))
        steps = dict((s['name'], s) for s in steps)

        self.assertNotIn('custom-step1', steps)
        self.assertIn('custom-step2', steps)

    def test_list_requires_inactive(self):
        def step1(registry):
            """ """
        def step2(registry):
            """ """
        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, requires=('custom-step2',), factory=step1)
        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        steps = Populate(self.registry).list_steps()
        d_steps = dict((s['name'], s) for s in steps)

        self.assertIn('custom-step1', d_steps)
        self.assertIn('custom-step2', d_steps)

    def test_list_requires_order(self):
        def step1(registry):
            """ """
        def step2(registry):
            """ """
        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, requires=('custom-step2',), factory=step1)
        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        steps = Populate(self.registry).list_steps()
        l_steps = [s['name'] for s in steps]

        self.assertTrue(l_steps.index('custom-step2') <
                        l_steps.index('custom-step1'))

    def test_list_once(self):
        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1', requires=('custom-step2',))
        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2')
        self.config.ptah_populate_step(
            'custom-step3', title='Custom step 3', requires=('custom-step2',))

        steps = Populate(self.registry).list_steps()

        count = 0
        for step in steps:
            if step['name'] == 'custom-step2':
                count += 1

        self.assertEqual(count, 1)

    def test_list_unknown(self):
        self.assertRaises(
            RuntimeError,
            Populate(self.registry).list_steps, ('unknown',))

    def test_list_unknown_dependency(self):
        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1', requires=('unknown',))

        self.assertRaises(
            RuntimeError, Populate(self.registry).list_steps)


class TestCreateDbSchema(ptah.PtahTestCase):

    def test_event(self):
        from ptah.populate import create_db_schema

        data = [False]
        def event_handler(ev):
            data[0] = True

        self.registry.registerHandler(
            event_handler, (ptah.events.BeforeCreateDbSchema,))

        create_db_schema(self.registry)
        self.assertTrue(data[0])

    def test_skip_tables(self):
        from ptah.populate import create_db_schema

        base = ptah.get_base()

        class test_populate_TestTable(base):
            __tablename__ = 'test_populate_TestTable'

            id = sqla.Column('id', sqla.Integer, primary_key=True)

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH)
        cfg['db_skip_tables'] = ('test_populate_TestTable',)

        create_db_schema(self.registry)

        self.assertFalse(
            base.metadata.tables['test_populate_TestTable'].exists())

        cfg['db_skip_tables'] = ()
        create_db_schema(self.registry)

        self.assertTrue(
            base.metadata.tables['test_populate_TestTable'].exists())



import ptah
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest
from pyramid.view import render_view_to_response
from pyramid.httpexceptions import HTTPFound


class TestSettingsModule(PtahTestCase):

    def test_settings_module(self):
        from ptah.manage.manage import PtahManageRoute
        from ptah.manage.settings import SettingsModule

        request = DummyRequest()

        ptah.auth_service.set_userid('test')
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['settings']

        self.assertIsInstance(mod, SettingsModule)

    def test_settings_view(self):
        from ptah.manage.settings import SettingsModule, SettingsView

        request = DummyRequest()

        mod = SettingsModule(None, request)

        res = render_view_to_response(mod, request, '', False)
        self.assertEqual(res.status, '200 OK')


class TestSettingsTTW(PtahTestCase):

    _init_ptah = False

    def test_traverse_ttw(self):
        from ptah.manage.settings import SettingsModule, SettingsWrapper

        ptah.register_settings(
            'test',
            ptah.form.TextField('node1', default='test1'),
            ptah.form.TextField('node2', default='test2', tint=True),
            title = 'Test settings')

        self.init_ptah()

        grp = ptah.get_settings('test', self.registry)
        mod = SettingsModule(None, self.request)

        self.assertRaises(
            KeyError,
            mod.__getitem__, 'test')

        grp.__ttw__ = True

        tgrp = mod['test']
        self.assertIs(tgrp.group, grp)
        self.assertIsInstance(tgrp, SettingsWrapper)

    def test_edit_form(self):
        from ptah.manage.settings import SettingsModule, EditForm

        ptah.register_settings(
            'test',
            ptah.form.TextField('node1', default='test1'),
            ptah.form.TextField('node2', default='test2'),
            title = 'Test settings',
            ttw = True,
            ttw_skip_fields = ('node2',))

        self.init_ptah()

        mod = SettingsModule(None, self.request)
        grp = mod['test']
        settings = grp.group

        form = EditForm(grp, self.request)

        self.assertEqual(form.label, settings.__title__)
        self.assertEqual(form.description, settings.__description__)
        self.assertIs(form.form_content(), settings)

        fields = form.fields
        self.assertIn('node1', fields)
        self.assertNotIn('node2', fields)

        res = form.back_handler()
        self.assertIsInstance(res, HTTPFound)
        self.assertEqual(res.headers['location'], '..')

        form.update()
        form.modify_handler()
        self.assertIn('Settings have been modified.',
                      ptah.render_messages(self.request))



import os
import sys
import shutil
import tempfile
import ptah
from ptah.scripts import migrate
from pyramid.compat import NativeIO


class TestMigrateCommand(ptah.PtahTestCase):

    _init_ptah = False

    def setUp(self):
        super(TestMigrateCommand, self).setUp()

        # fix stdout
        self.orig_stdout = sys.stdout
        self.out = out = NativeIO()
        sys.stdout = out

        # fix bootstrap
        import ptah.scripts
        self.original_bootstrap = ptah.scripts.bootstrap

        def bootstrap(*args, **kw):
            return {'registry': self.registry, 'request': self.request,
                    'app': object()}

        ptah.scripts.bootstrap = bootstrap

        # fix ScriptDirectory
        from ptah import migrate

        self.dirs = dirs = {}

        class ScriptDirectory(migrate.ScriptDirectory):

            def __init__(self, pkg):
                if pkg in dirs:
                    dir = dirs[pkg]
                else:
                    dir = tempfile.mkdtemp()
                    dirs[pkg] = dir

                self.dir = os.path.join(
                    os.path.dirname(ptah.__file__), 'scripts')
                self.versions = dir

        self.orig_ScriptDirectory = migrate.ScriptDirectory
        migrate.ScriptDirectory = ScriptDirectory

    def tearDown(self):
        # reset bootstrap
        import ptah.scripts
        ptah.scripts.bootstrap = self.original_bootstrap

        # reset stdout
        sys.stdout = self.orig_stdout

        # reset ScriptDirectory
        for dir in self.dirs.values():
            shutil.rmtree(dir)

        from ptah import migrate
        migrate.ScriptDirectory = self.orig_ScriptDirectory

        super(TestMigrateCommand, self).tearDown()

    def _reset_stdout(self):
        sys.stdout = self.orig_stdout

    def test_script_help(self):
        self.init_ptah()

        sys.argv[:] = ['ptah-migrate', '-h', 'ptah.ini']

        try:
            migrate.main()
        except:
            pass

        self._reset_stdout()
        self.assertIn('usage: ptah-migrate [-h] config', self.out.getvalue())

    def test_list_migrations(self):
        ptah.register_migration('test', 'test:path', 'Test migration')

        self.init_ptah()

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'list']

        migrate.main()

        self._reset_stdout()
        self.assertIn('* test: Test migration', self.out.getvalue())

    def test_upgrade_one(self):
        from ptah.migrate import revision, Version

        ptah.register_migration('test1', 'test1:path', 'Test migration',True)
        ptah.register_migration('test2', 'test2:path', 'Test migration',True)

        self.init_ptah()

        rev1 = revision('test1')
        rev2 = revision('test2')

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'upgrade', 'test1']

        migrate.main()

        self._reset_stdout()

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())

        self.assertIn('test1', versions)
        self.assertEqual(versions['test1'], rev1)
        self.assertNotIn('test2', versions)

    def test_upgrade_several(self):
        from ptah.migrate import revision, Version

        ptah.register_migration('test1', 'test1:path', 'Test migration')
        ptah.register_migration('test2', 'test2:path', 'Test migration')

        self.init_ptah()

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())

        rev1 = revision('test1')
        rev2 = revision('test2')

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'upgrade', 'test1', 'test2']

        migrate.main()

        self._reset_stdout()

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())

        self.assertIn('test1', versions)
        self.assertIn('test2', versions)
        self.assertEqual(versions['test1'], rev1)
        self.assertEqual(versions['test2'], rev2)

    def test_revision(self):
        from ptah.migrate import revision, Version

        ptah.register_migration('test', 'test:path', 'Test migration')
        self.init_ptah()

        sys.argv[:] = ['ptah-migrate', 'ptah.ini',
                       'revision', 'test', '-r', '001', '-m', 'Test message']
        migrate.main()

        path = self.dirs['test']
        self.assertIn('001.py', os.listdir(path))

    def test_revision_error(self):
        from ptah.migrate import revision, Version

        ptah.register_migration('test', 'test:path', 'Test migration')
        self.init_ptah()

        sys.argv[:] = ['ptah-migrate', 'ptah.ini',
                       'revision', 'test', '-r', '0.0;1', '-m', 'Test message']
        migrate.main()
        self._reset_stdout()

        self.assertIn('Revision id contains forbidden characters',
                      self.out.getvalue())

    def test_history(self):
        from ptah.migrate import revision, upgrade, Version

        ptah.register_migration('test1', 'test1:path', 'Test migration')
        ptah.register_migration('test2', 'test2:path', 'Test migration')

        self.init_ptah()

        rev1 = revision('test1', message='test1 step')
        rev2 = revision('test2', message='test2 step')

        upgrade('test1')
        upgrade('test2')

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'history', 'test1']

        migrate.main()

        self.assertIn('test1 step', self.out.getvalue())

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'history']
        migrate.main()

        self.assertIn('test1 step', self.out.getvalue())
        self.assertIn('test2 step', self.out.getvalue())

    def test_current_one(self):
        from ptah.migrate import revision, upgrade, Version

        ptah.register_migration('test1', 'test1:path', 'Test migration')
        ptah.register_migration('test2', 'test2:path', 'Test migration')

        self.init_ptah()

        rev1 = revision('test1', message='test1 step')
        rev2 = revision('test2', message='test2 step')
        upgrade('test1')

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'current', 'test1']

        migrate.main()

        self.assertIn("Package 'test1' rev: %s(head) test1 step"%rev1,
                      self.out.getvalue())

    def test_current_all(self):
        from ptah.migrate import revision, upgrade, Version

        ptah.register_migration('test1', 'test1:path', 'Test migration')
        ptah.register_migration('test2', 'test2:path', 'Test migration')

        self.init_ptah()

        rev1 = revision('test1', message='test1 step')
        rev2 = revision('test2', message='test2 step')
        upgrade('test1')

        sys.argv[:] = ['ptah-migrate', 'ptah.ini', 'current']

        migrate.main()

        self.assertIn("Package 'test1' rev: %s(head) test1 step"%rev1,
                      self.out.getvalue())
        self.assertIn("Package 'test2' rev: None", self.out.getvalue())



from pyramid.security import authenticated_userid
from pyramid.threadlocal import get_current_request

import ptah
from ptah import config
from ptah.uri import resolve, resolver
from ptah.util import tldata


class _Superuser(object):
    """ Default ptah superuser. check_permission always pass with superuser """

    def __init__(self):
        self.__uri__ = 'ptah-auth:superuser'
        self.login = ''
        self.name = 'Manager'

    def __repr__(self):
        return '<ptah Superuser>'


SUPERUSER = _Superuser()
SUPERUSER_URI = 'ptah-auth:superuser'


@resolver('ptah-auth')
def superuser_resolver(uri):
    """System super user"""
    if uri == SUPERUSER_URI:
        return SUPERUSER


AUTH_CHECKER_ID = 'ptah:authchecker'
AUTH_PROVIDER_ID = 'ptah:authprovider'
AUTH_SEARCHER_ID = 'ptah:authsearcher'


def auth_checker(checker, __cfg=None, __depth=1):
    """Register authentication checker. Checker function accepts
    :py:class:`ptah.authentication.AuthInfo` object.

    :param checker: Checker function.

    Checker function interface :py:func:`ptah.interfaces.auth_checker`

    .. code-block:: python

      @ptah.auth_checker
      def my_checker(info):
          ...

    """
    info = config.DirectiveInfo(__depth)
    discr = (AUTH_CHECKER_ID, hash(checker))
    intr = config.Introspectable(
        AUTH_CHECKER_ID, discr, checker.__name__, AUTH_CHECKER_ID)
    intr['name'] = '{0}.{1}'.format(info.codeinfo.module, checker.__name__)
    intr['callable'] = checker
    intr['codeinfo'] = info.codeinfo

    info.attach(
        config.Action(
            lambda config, checker: config.get_cfg_storage(AUTH_CHECKER_ID)\
                .update({id(checker): checker}),
            (checker,), discriminator=discr, introspectables=(intr,)),
        __cfg)
    return checker


def pyramid_auth_checker(cfg, checker):
    """ Pyramid configurator directive for authentication
    checker registration.

    :param checker: Checker function

    Checker function interface :py:func:`ptah.interfaces.auth_checker`

    .. code-block:: python

      config = Configurator()
      config.include('ptah')

      def my_checker(info):
          ...

      config.ptah_auth_checker(my_checker)
    """
    auth_checker(checker, cfg, 3)


class auth_provider(object):
    """ Register authentication provider.
    Auth provider interface :py:class:`ptah.interfaces.AuthProvider`

    :param name: provider name

    .. code-block:: python

      @ptah.auth_provider('my-provider')
      class AuthProvider(object):
           ...

    """
    def __init__(self, name, __depth=1):
        self.info = config.DirectiveInfo(__depth)

        self.discr = (AUTH_PROVIDER_ID, name)
        self.intr = config.Introspectable(
            AUTH_PROVIDER_ID, self.discr, name, AUTH_PROVIDER_ID)
        self.intr['id'] = name
        self.intr['codeinfo'] = self.info.codeinfo

    def __call__(self, cls, __cfg=None):
        self.intr['provider'] = cls
        self.intr['name'] = '{0}.{1}'.format(
            self.info.codeinfo.module, cls.__name__)

        self.info.attach(
            config.Action(
                lambda config, n, p: config.get_cfg_storage(AUTH_PROVIDER_ID)\
                    .update({n: cls()}),
                (self.intr['name'], cls),
                discriminator=self.discr, introspectables=(self.intr,)),
            __cfg)
        return cls

    @classmethod
    def register(cls, name, provider):
        """ authentication provider registration::

        .. code-block:: python

          class AuthProvider(object):
             ...

          ptah.auth_provider.register('my-provider', AuthProvider)

        """
        cls(name, 2)(provider)

    @classmethod
    def pyramid(cls, cfg, name, provider):
        """ ``ptah_auth_provider`` directive implementation """
        cls(name, 3)(provider, cfg)


class AuthInfo(object):
    """ Authentication information """

    #: Principal uri or None if principal is not set
    __uri__ = None

    #: Principal object
    principal = None

    #: Status, True is principal has been authenticated, false otherwise
    status = False

    #: Extra message from auth checkers
    message = False

    def __init__(self, principal, status=False, message=''):
        self.__uri__ = getattr(principal, '__uri__', None)
        self.principal = principal
        self.status = status
        self.message = message
        self.arguments = {}


_not_set = object()

USER_KEY = '__ptah_userid__'
EFFECTIVE_USER_KEY = '__ptah_effective__userid__'


class Authentication(object):
    """ Ptah authentication utility """

    def authenticate(self, credentials):
        """Authenticate credentials.

        :param credentials: Dictionary with `login` and `password`
        :rtype: :py:class:`ptah.authentication.AuthInfo`
        """
        providers = config.get_cfg_storage(AUTH_PROVIDER_ID)
        for pname, provider in providers.items():
            principal = provider.authenticate(credentials)
            if principal is not None:
                info = AuthInfo(principal)

                for checker in \
                        config.get_cfg_storage(AUTH_CHECKER_ID).values():
                    if not checker(info):
                        return info

                info.status = True
                return info

        return AuthInfo(None)

    def authenticate_principal(self, principal):
        """Authenticate principal, check principal with
        auth checkers

        :param principal: Principal object
        :rtype: :py:class:`ptah.authentication.AuthInfo`
        """
        info = AuthInfo(principal)

        for checker in \
                config.get_cfg_storage(AUTH_CHECKER_ID).values():
            if not checker(info):
                return info

        info.status = True
        return info

    def set_userid(self, uri):
        """ Set current user id """
        tldata.set(USER_KEY, uri)

    def get_userid(self):
        """ Get current user id. By default it uses
        ``pyramid.security.authenticated_userid``"""
        uri = tldata.get(USER_KEY, _not_set)
        if uri is _not_set:
            self.set_userid(authenticated_userid(get_current_request()))
            return tldata.get(USER_KEY)
        return uri

    def set_effective_userid(self, uri):
        """ Set effective user uri """
        tldata.set(EFFECTIVE_USER_KEY, uri)

    def get_effective_userid(self):
        """ Return effective user uri, of current user uri. """
        uri = tldata.get(EFFECTIVE_USER_KEY, _not_set)
        if uri is _not_set:
            return self.get_userid()
        return uri

    def get_current_principal(self):
        """ Resolve and return current user uri """
        return resolve(self.get_userid())

    def get_principal_bylogin(self, login):
        """ Return principal by login """
        providers = config.get_cfg_storage(AUTH_PROVIDER_ID)

        for pname, provider in providers.items():
            principal = provider.get_principal_bylogin(login)
            if principal is not None:
                return principal

auth_service = Authentication()


def search_principals(term):
    """ Search principals by term, it uses principal_searcher functions """
    searchers = config.get_cfg_storage(AUTH_SEARCHER_ID)
    for name, searcher in searchers.items():
        for principal in searcher(term):
            yield principal


class principal_searcher(object):
    """ Register principal searcher function.

    Searcher function interface :py:func:`ptah.interfaces.principal_searcher`

    .. code-block:: python

      @ptah.principal_searcher('test')
      def searcher(term):
           ...

    searcher function receives text as term variable, and
    should return iterator to principal objects.
    """
    def __init__(self, name, __depth=1):
        self.info = config.DirectiveInfo(__depth)

        self.discr = (AUTH_SEARCHER_ID, name)
        self.intr = config.Introspectable(
            AUTH_SEARCHER_ID, self.discr, name, AUTH_SEARCHER_ID)
        self.intr['name'] = name

    def __call__(self, searcher, cfg=None):
        self.intr['callable'] = searcher

        self.info.attach(
            config.Action(
                lambda config, name, searcher:
                    config.get_cfg_storage(AUTH_SEARCHER_ID)\
                        .update({name: searcher}),
                (self.intr['name'], searcher),
                discriminator=self.discr, introspectables=(self.intr,)),
            cfg)

        return searcher

    @classmethod
    def register(cls, name, searcher):
        """ register principal searcher:

        .. code-block:: python

          def searcher(term):
              ...

          ptah.principal_searcher.register('test', searcher)

        """
        cls(name, 2)(searcher)

    @classmethod
    def pyramid(cls, cfg, name, searcher):
        """ pyramid configurator directive for
        principal searcher registration """
        cls(name, 3)(searcher, cfg)



import os
import sys
import logging
import multiprocessing # atexit exception
from setuptools import setup, find_packages

version='0.3.2'

install_requires = ['setuptools',
                    'alembic',
                    'chameleon >= 2.6.1',
                    'pyramid >= 1.3a5',
                    'pyramid_tm',
                    'zope.interface >= 3.8.0',
                    'zope.sqlalchemy >= 0.7.0',
                    'transaction >= 1.2.0',
                    'venusian',
                    'WebOb >= 1.2b2',
                    'SQLAlchemy',
                    'Pygments',
                    'pytz',
                    'sphinx',
                    ]

if sys.version_info[:2] == (2, 6):
    install_requires.extend((
            'argparse',
            'ordereddict',
            'unittest2'))

tests_require = install_requires + ['nose']

def read(f):
    return open(os.path.join(os.path.dirname(__file__), f)).read().strip()


setup(name='ptah',
      version=version,
      description=('Ptah is a fast, fun, open source high-level '
                   'Python web development environment.'),
      long_description='\n\n'.join((read('README.rst'), read('CHANGES.txt'))),
      classifiers=[
          "Intended Audience :: Developers",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.2",
          "Programming Language :: Python :: Implementation :: CPython",
          "Framework :: Pylons",
          "Topic :: Internet :: WWW/HTTP",
          'Topic :: Internet :: WWW/HTTP :: WSGI'],
      author='Ptah Project',
      author_email='ptahproject@googlegroups.com',
      url='https://github.com/ptahproject/ptah/',
      license='BSD-derived',
      packages=find_packages(),
      install_requires = install_requires,
      tests_require = tests_require,
      test_suite = 'nose.collector',
      include_package_data = True,
      zip_safe = False,
      entry_points = {
          'console_scripts': [
              'ptah-manage = ptah.scripts.manage:main',
              'ptah-migrate = ptah.scripts.migrate:main',
              'ptah-populate = ptah.scripts.populate:main',
              'ptah-settings = ptah.scripts.settings:main',
              ],
          'pyramid.scaffold': [
              'ptah_starter = ptah.scaffolds:PtahStarterProjectTemplate',
              ],
          },
      package_data = {'migrations': ['ptah/migrations/*.py']},
      )



import os
import logging
import functools
import pkg_resources
import sqlalchemy as sqla

import alembic.util
import alembic.config
import alembic.context
from alembic.script import ScriptDirectory

import ptah
from ptah import config
from pyramid.path import package_name, AssetResolver

MIGRATION_ID = 'ptah:migrate'


class Version(ptah.get_base()):

    __tablename__ = 'ptah_db_versions'

    package = sqla.Column(sqla.String(128), primary_key=True)
    version_num = sqla.Column(sqla.String(32), nullable=False)


class ScriptDirectory(ScriptDirectory):

    def __init__(self, pkg):
        path = ptah.get_cfg_storage(MIGRATION_ID).get(pkg)
        if path is None:
            raise ValueError("Can't find package.")

        res = AssetResolver(pkg)
        self.dir = res.resolve('ptah:scripts').abspath()
        self.versions = res.resolve(path).abspath()

        if not os.access(self.versions, os.F_OK):
            raise alembic.util.CommandError("Path doesn't exist: %r." % path)


class Context(alembic.context.Context):

    pkg_name = ''

    def _current_rev(self):
        if self.as_sql: # pragma: no cover
            return self._start_from_rev
        else:
            if self._start_from_rev: # pragma: no cover
                raise alembic.util.CommandError(
                    "Can't specify current_rev to context "
                    "when using a database connection")
            Version.__table__.create(checkfirst=True)

        item = ptah.get_session().query(Version.version_num).filter(
            Version.package == self.pkg_name).first()
        return getattr(item, 'version_num', None)

    def _update_current_rev(self, old, new):
        if old == new: # pragma: no cover
            return
        if new is None: # pragma: no cover
            self.impl._exec(Version.__table__.delete().where(
                package=self.pkg_name))
        elif old is None:
            self.impl._exec(
                Version.__table__.insert().
                values(package=self.pkg_name,
                       version_num=sqla.literal_column("'%s'" % new))
                )
        else:
            self.impl._exec(
                Version.__table__.update().
                values(package=self.pkg_name,
                       version_num=sqla.literal_column("'%s'" % new))
                )

    def run_migrations(self, **kw):
        current_rev = rev = False
        log = logging.getLogger('ptah.alembic')
        self.impl.start_migrations()

        for change, prev_rev, rev in self._migrations_fn(self._current_rev()):
            if current_rev is False:
                current_rev = prev_rev
                if self.as_sql and not current_rev: # pragma: no cover
                    Version.__table__.create(checkfirst=True)

            log.info("%s: running %s %s -> %s",
                     self.pkg_name, change.__name__, prev_rev, rev)
            if self.as_sql: # pragma: no cover
                self.impl.static_output(
                    "-- Running %s %s -> %s"%(change.__name__, prev_rev, rev))

            change(**kw)

            if not self.impl.transactional_ddl: # pragma: no cover
                self._update_current_rev(prev_rev, rev)
            prev_rev = rev

        if rev is not False:
            if self.impl.transactional_ddl:
                self._update_current_rev(current_rev, rev)

            if self.as_sql and not rev: # pragma: no cover
                Version.__table___.drop()


def upgrade(pkg, sql=False):
    """Upgrade to a later version."""
    if ':' in pkg:
        pkg, rev = pkg.split(':',1)
    else:
        rev = 'head'

    script = ScriptDirectory(pkg)

    alembic.context.Context = Context
    alembic.context.Context.pkg_name = pkg
    alembic.context._opts(
        alembic.config.Config(''),
        script,
        fn = functools.partial(script.upgrade_from, rev),
        as_sql = sql,
        starting_rev = None,
        destination_rev = rev,
    )

    conn = ptah.get_base().metadata.bind.connect()

    alembic.context.configure(
        connection=conn,
        config=alembic.config.Config(''))

    with alembic.context.begin_transaction():
        alembic.context.run_migrations()


def revision(pkg, rev=None, message=None):
    """Create a new revision file."""
    script = ScriptDirectory(pkg)
    revs = [sc.revision for sc in script.walk_revisions()]

    if not rev:
        rev = alembic.util.rev_id()

    if rev in revs:
        raise KeyError('Revision already exists')

    return script.generate_rev(rev, message)


def ptah_migrate(cfg):
    def action(cfg,):
        for pkg in cfg.get_cfg_storage(MIGRATION_ID).keys():
            upgrade(pkg)

    cfg.action('ptah.ptah_migrate', action, (cfg,), order=999999+1)


def register_migration(pkg, path, title='', force=False):
    """Registers a migration for package.
    Check :ref:`data_migration_chapter` chapter for detailed description.

    :param pkg: Package name
    :param path: String implying a path or `asset specification`
        (e.g. ``ptah:migrations``). Path to directory with migration scripts.
    :param title: Optional human readable title.
    :param force: Force execute migration during bootstrap process

    .. code-block:: python

      import ptah

      ptah.register_migration(
          'ptah', 'ptah:migrations', 'Ptah database migration')

    """
    info = config.DirectiveInfo()
    discr = (MIGRATION_ID, pkg)

    intr = config.Introspectable(MIGRATION_ID, discr, pkg, MIGRATION_ID)
    intr['package'] = pkg
    intr['path'] = path
    intr['title'] = title
    intr['force'] = force

    def _complete(cfg, pkg, path):
        cfg.get_cfg_storage(MIGRATION_ID)[pkg] = path

    info.attach(
        config.Action(
            _complete, (pkg, path),
            discriminator=discr, introspectables=(intr,))
        )


def update_versions(registry):
    packages = []
    for item in registry.introspector.get_category(MIGRATION_ID):
        intr = item['introspectable']
        if not intr['force']:
            packages.append(intr['package'])

    session = ptah.get_session()

    for pkg in packages:
        item = session.query(Version).filter(Version.package==pkg).first()
        if item is not None:
            continue

        script = ScriptDirectory(pkg)
        revs = [sc for sc in script.walk_revisions()]

        # set head as version
        for sc in revs:
            if sc.is_head:
                session.add(Version(package=pkg, version_num=sc.revision))
                break


def check_version(ev):
    """ ApplicationCreated event handler """
    if not Version.__table__.exists():
        return

    versions = dict((v.package, v.version_num)
                    for v in ptah.get_session().query(Version).all())
    packages = ptah.get_cfg_storage(MIGRATION_ID).keys()

    has_steps = False
    log = logging.getLogger('ptah.alembic')

    for pkg in packages:
        version = versions.get(pkg)
        script = ScriptDirectory(pkg)
        for sc in script.walk_revisions():
            if sc.is_head:
                if sc.revision != version:
                    has_steps = True
                    log.error("Package '%s' current revision: '%s', head: '%s'",
                              pkg, version, sc.revision)
                break

    if has_steps:
        config.shutdown()
        log.error("Please run `ptah-migrate` script. Stopping...")
        raise SystemExit(1)



import os
from pyramid.scaffolds.template import Template


class PtahTemplate(Template):

    def post(self, command, output_dir, vars): # pragma: no cover
        print ("\n\nPtah has generated your application. Your application "
               "can be found at /. \n\nDorneles hopes you like it.\n")

        return super(PtahTemplate, self).post(command, output_dir, vars)


class PtahStarterProjectTemplate(PtahTemplate):
    _template_dir = 'ptah_starter'
    summary = 'Ptah Starter - blank Ptah scaffold'



import json
from threading import local

import sqlalchemy as sqla
from sqlalchemy import orm
from sqlalchemy.ext import declarative
from sqlalchemy.ext.mutable import Mutable
from sqlalchemy.types import TypeDecorator, TEXT
from zope.sqlalchemy import ZopeTransactionExtension

_base = declarative.declarative_base()
_zte = ZopeTransactionExtension()
_session = orm.scoped_session(orm.sessionmaker(extension=[_zte]))


def get_base():
    """Return the central SQLAlchemy declarative base."""
    return _base


def reset_session():
    """Reset sqla session"""
    global _zte, _session

    _zte = ZopeTransactionExtension()
    _session = orm.scoped_session(orm.sessionmaker(extension=[_zte]))


def get_session():
    """Return the central SQLAlchemy contextual session.

    To customize the kinds of sessions this contextual session creates, call
    its ``configure`` method::

        ptah.get_session().configure(...)

    But if you do this, be careful about the 'ext' arg. If you pass it, the
    ZopeTransactionExtension will be disabled and you won't be able to use this
    contextual session with transaction managers. To keep the extension active
    you'll have to re-add it as an argument. The extension is accessible under
    the semi-private variable ``_zte``. Here's an example of adding your own
    extensions without disabling the ZTE::

        ptah.get_session().configure(ext=[ptah._zte, ...])
    """
    return _session


class QueryFreezer(object):
    """ A facade for sqla.Session.query which caches internal query structure.

    :param builder: anonymous function containing SQLAlchemy query

    .. code-block:: python

        _sql_parent = ptah.QueryFreezer(
            lambda: Session.query(Content)
                .filter(Content.__uri__ == sqla.sql.bindparam('parent')))
    """
    _testing = False

    def __init__(self, builder):
        self.builder = builder
        self.data = local()

    def reset(self):
        self.data = local()

    def iter(self, **params):
        data = self.data
        if not hasattr(data, 'query') or self._testing:
            data.query = self.builder()
            data.mapper = data.query._mapper_zero_or_none()
            data.querycontext = data.query._compile_context()
            data.querycontext.statement.use_labels = True
            data.stmt = data.querycontext.statement

        conn = data.query._connection_from_session(
            mapper=data.mapper,
            clause=data.stmt,
            close_with_result=True)

        result = conn.execute(data.stmt, **params)
        return data.query.instances(result, data.querycontext)

    def one(self, **params):
        ret = list(self.iter(**params))

        l = len(ret)
        if l == 1:
            return ret[0]
        elif l == 0:
            raise orm.exc.NoResultFound("No row was found for one()")
        else:
            raise orm.exc.MultipleResultsFound(
                "Multiple rows were found for one()")

    def first(self, **params):
        ret = list(self.iter(**params))[0:1]
        if len(ret) > 0:
            return ret[0]
        else:
            return None

    def all(self, **params):
        return list(self.iter(**params))


class JsonType(TypeDecorator):
    """Represents an immutable structure as a json-encoded string."""

    impl = TEXT

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = json.dumps(value)
        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = json.loads(value)
        return value


class MutationList(Mutable, list):

    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, MutationList):
            if isinstance(value, list):
                return MutationList(value)
            return Mutable.coerce(key, value)  # pragma: no cover
        else:
            return value

    def append(self, value):
        list.append(self, value)
        self.changed()

    def __setitem__(self, key, value):
        list.__setitem__(self, key, value)
        self.changed()

    def __delitem__(self, key):
        list.__delitem__(self, key)
        self.changed()


class MutationDict(Mutable, dict):

    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, MutationDict):
            if isinstance(value, dict):
                return MutationDict(value)
            return Mutable.coerce(key, value)  # pragma: no cover
        else:
            return value

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        self.changed()

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        self.changed()


def JsonDictType():
    """
    function which returns a SQLA Column Type suitable to store a Json dict.

    :returns: ptah.sqla.MutationDict
    """
    return MutationDict.as_mutable(JsonType)


def JsonListType():
    """
    function which returns a SQLA Column Type suitable to store a Json array.

    :returns: ptah.sqla.MutationList
    """

    return MutationList.as_mutable(JsonType)



""" settings """
import logging
import os.path
import sqlalchemy as sqla
from collections import OrderedDict

from zope import interface
from zope.interface.interface import InterfaceClass
from pyramid.compat import configparser
from pyramid.interfaces import PHASE1_CONFIG

import ptah
from ptah import uri, form, config
from ptah.config import StopException

log = logging.getLogger('ptah')

SETTINGS_ID = 'settings'
SETTINGS_OB_ID = 'ptah:settings'
ID_SETTINGS_GROUP = 'ptah:settings-group'

_marker = object()


def get_settings(grp, registry=None):
    """Get settings group by group id. Also there is `ptah_get_settins`
    pyramid configurator directive.

    .. code-block:: python

      config = Configurator()
      config.include('ptah')
      config.commit()

      # get settings with pyramid directive
      ptah_settings = config.ptah_get_settings('ptah')

      # get settings with `get_settings`
      ptah_settings = ptah.get_settings(ptah.CFG_ID_PTAH, config.registry)

    """
    return config.get_cfg_storage(ID_SETTINGS_GROUP, registry)[grp]


@uri.resolver('settings')
def settings_resolver(uri):
    """ Ptah settings resolver """
    return config.get_cfg_storage(ID_SETTINGS_GROUP)[uri[9:]]


def pyramid_get_settings(config, grp):
    """ pyramid configurator directive for getting settings group::

        config = Configurator()
        config.include('ptah')

        PTAH_CFG = config.get_settings(ptah.CFG_ID_PTAH)
    """
    return config.get_cfg_storage(ID_SETTINGS_GROUP)[grp]


def init_settings(pconfig, cfg=None, section=configparser.DEFAULTSECT):
    """Initialize settings management system. This function available
    as pyramid configurator directive. You should call it during
    application configuration process.

    .. code-block:: python

      config = Configurator()
      config.include('ptah')

      # initialize ptah setting management system
      config.ptah_init_settings()

    """
    registry = pconfig.registry
    settings = config.get_cfg_storage(SETTINGS_OB_ID,pconfig.registry,Settings)

    if settings.initialized:
        raise RuntimeError(
            "initialize_settings has been called more than once.")

    log.info('Initializing ptah settings')

    settings.initialized = True

    if cfg is None:
        cfg = pconfig.registry.settings

    here = cfg.get('here', './')
    include = cfg.get('include', '')
    for f in include.split('\n'):
        f = f.strip()
        if f and os.path.exists(f):
            parser = configparser.SafeConfigParser()
            parser.read(f)
            if section == configparser.DEFAULTSECT or \
                    parser.has_section(section):
                cfg.update(parser.items(section, vars={'here': here}))

    pconfig.begin()
    try:
        settings.init(pconfig, cfg)
        pconfig.registry.notify(
            ptah.events.SettingsInitializing(pconfig, pconfig.registry))
        pconfig.registry.notify(
            ptah.events.SettingsInitialized(pconfig, pconfig.registry))
    except Exception as e:
        raise StopException(e)
    finally:
        pconfig.end()


def register_settings(name, *fields, **kw):
    """Register settings group.

    :param name: Name of settings group
    :param fields: List of :py:class:`ptah.form.Field` objects

    """
    iname = name
    for ch in ('.', '-'):
        iname = iname.replace(ch, '_')

    category = InterfaceClass(
        'SettingsGroup:%s' % iname.upper(), (),
        __doc__='Settings group: %s' % name,
        __module__='ptah.config.settings')

    for field in fields:
        field.required = False
        field.missing = field.default
        if field.default is form.null:
            raise StopException(
              'Default value is required for "{0}.{1}"'.format(name,field.name))

    group = Group(name=name, *fields, **kw)
    interface.directlyProvides(Group, category)

    info = config.DirectiveInfo()
    discr = (ID_SETTINGS_GROUP, name)
    intr = config.Introspectable(
        ID_SETTINGS_GROUP, discr, group.__title__, ID_SETTINGS_GROUP)
    intr['name'] = name
    intr['group'] = group
    intr['codeinfo'] = info.codeinfo

    info.attach(
        config.Action(
            lambda config, group: config.get_cfg_storage(ID_SETTINGS_GROUP)\
                .update({group.__name__: group.clone(config.registry)}),
            (group,), discriminator=discr, introspectables=(intr,))
        )


class Settings(object):
    """ settings management system """

    initialized = False

    def init(self, config, defaults=None):
        groups = config.get_cfg_storage(ID_SETTINGS_GROUP).items()

        for name, group in groups:
            data = {}
            for field in group.__fields__.values():
                if field.default is not form.null:
                    data[field.name] = field.default

            group.update(data)

        if defaults is None: # pragma: no cover
            return

        self.load(defaults, True)

    def load(self, rawdata, setdefaults=False):
        groups = config.get_cfg_storage(ID_SETTINGS_GROUP).items()

        try:
            rawdata = dict((k.lower(), v) for k, v in rawdata.items())
        except Exception as e:
            raise StopException(e)

        for name, group in groups:
            data, errors = group.extract(rawdata)

            if errors:
                log.error(errors.msg)
                raise StopException(errors)

            for k, v in data.items():
                if v is not form.null:
                    group.__fields__[k].default = v

            group.update(data)

    def load_fromdb(self):
        self.load(dict(ptah.get_session().\
                           query(SettingRecord.name, SettingRecord.value)))

    def export(self, default=False):
        groups = config.get_cfg_storage(ID_SETTINGS_GROUP).items()

        result = {}
        for name, group in groups:
            for field in group.__fields__.values():
                fname = field.name
                if group[fname] == field.default and not default:
                    continue

                result['{0}.{1}'.format(name,fname)] = field.dumps(group[fname])

        return result


class Group(OrderedDict):
    """ Settings group """

    def __init__(self, *args, **kwargs):
        super(Group, self).__init__()

        fields = form.Fieldset(*args, **kwargs)
        self.__uri__ = 'settings:{0}'.format(fields.name)
        self.__name__ = fields.name
        self.__title__ = fields.title
        self.__description__ = fields.description
        self.__fields__ = fields
        self.__ttw__ = kwargs.get('ttw', False)
        self.__ttw_skip_fields__ = kwargs.get('ttw_skip_fields', ())

    def clone(self, registry):
        clone = self.__class__.__new__(self.__class__)
        clone.__dict__.update(self.__dict__)
        clone.__registry__ = registry
        return clone

    def extract(self, rawdata):
        fieldset = self.__fields__
        name = fieldset.name

        data = {}
        errors = form.FieldsetErrors(fieldset)

        for field in fieldset.fields():
            value = rawdata.get('{0}.{1}'.format(name, field.name), _marker)

            if value is _marker:
                value = self.get(field.name)
            else:
                try:
                    value = field.loads(value)
                    field.validate(value)

                    if field.preparer is not None:
                        value = field.preparer(value)
                except form.Invalid as e:
                    errors.append(e)
                    value = field.default

            data[field.name] = value

        if not errors:
            try:
                fieldset.validate(data)
            except form.Invalid as e:
                errors.append(e)

        return data, errors

    def get(self, name, default=None):
        try:
            return super(Group, self).__getitem__(name)
        except (KeyError, AttributeError):
            pass

        if name in self.__fields__:
            return self.__fields__[name].default

        return default

    def keys(self):
        return [node.name for node in self.__fields__.values()]

    def items(self):
        return [(key, self.get(key)) for key in self.__fields__.keys()]

    def __getitem__(self, name):
        res = self.get(name, _marker)
        if res is _marker:
            raise KeyError(name)
        return res

    def updatedb(self, **data):
        self.update(data)

        name = self.__name__
        fields = self.__fields__

        Session = ptah.get_session()

        # remove old data
        keys = tuple('{0}.{1}'.format(name, key) for key in data.keys())
        if keys:
            Session.query(SettingRecord)\
                .filter(SettingRecord.name.in_(keys)).delete(False)

        # insert new data
        result = []
        for fname in data.keys():
            if fname not in fields:
                continue

            field = fields[fname]
            value = self[fname]
            if value == field.default:
                continue

            result.append(
                {'name': '{0}.{1}'.format(name,fname),
                 'value': field.dumps(value)})
            Session.add(
                SettingRecord(name='{0}.{1}'.format(name,fname),
                              value=field.dumps(value)))
        Session.flush()

        self.__registry__.notify(ptah.events.SettingsGroupModified(self))
        self.__registry__.notify(ptah.events.UriInvalidateEvent(self.__uri__))


class SettingRecord(ptah.get_base()):

    __tablename__ = 'ptah_settings'

    name = sqla.Column(sqla.String(128), primary_key=True)
    value = sqla.Column(sqla.UnicodeText)



import ptah
from pyramid.view import view_config
from pyramid.interfaces import IRouteRequest


@ptah.manage.module('rest')
class RestModule(ptah.manage.PtahModule):
    """
    REST Introspector
    """

    title = 'REST Introspector'

    def available(self):
        """ check if ptah rest api is enabled """
        return self.request.registry.queryUtility(
            IRouteRequest, 'ptah-rest') is not None


@view_config(
    context=RestModule,
    wrapper=ptah.wrap_layout(),
    renderer='ptah.manage:templates/rest.pt')

class RestModuleView(ptah.View):

    def update(self):
        self.url = self.request.params.get(
            'url', '{0}/__rest__/cms/'.format(self.application_url))



import ptah
from pyramid.exceptions import ConfigurationConflictError


class TestEvent(ptah.PtahTestCase):

    _init_ptah = False

    def test_event_registration(self):
        import ptah

        @ptah.event('TestEvent')
        class TestEvent(object):
            """ test event """

        self.init_ptah()

        storage = self.config.get_cfg_storage(ptah.event.ID_EVENT)

        self.assertIn(TestEvent, storage)
        ev = storage[TestEvent]
        self.assertIn(ev.name, storage)

    def test_event_registration_dupl(self):
        import ptah

        @ptah.event('TestEvent')
        class TestEvent1(object):
            """ test event """

        @ptah.event('TestEvent')
        class TestEvent2(object):
            """ test event """

        self.config.scan('ptah')
        self.config.scan(self.__class__.__module__)
        self.assertRaises(ConfigurationConflictError, self.init_ptah)

    def test_event_intr(self):
        import ptah

        @ptah.event('TestEvent')
        class TestEvent(object):
            """ test event """

        self.init_ptah()

        name = '{0}.{1}'.format(TestEvent.__module__, TestEvent.__name__)

        intr = self.registry.introspector.get(
            ptah.event.ID_EVENT,
            (ptah.event.ID_EVENT, name))
        self.assertIsNotNone(intr)
        self.assertIs(intr['ev'].factory, TestEvent)
